==> spring-boot-ext-1.2.284-20190719.015755-4-sources.jar: cn/com/duibaboot/ext/autoconfigure/flowreplay/replay/ReplayMockSupport.java <==
package cn.com.duibaboot.ext.autoconfigure.flowreplay.replay;

import cn.com.duibaboot.ext.autoconfigure.flowreplay.FlowReplayConstants;
import cn.com.duibaboot.ext.autoconfigure.flowreplay.FlowReplayUtils;
import mockit.Mock;
import mockit.MockUp;
import mockit.internal.startup.Startup;
import net.bytebuddy.agent.ByteBuddyAgent;

import java.util.UUID;

/**
 * 此类用于返回模拟的方法值，目前用于 System.currentTimeMillis()、new Date()、UUID.randomUUID().toString() 等调用，这个类仅供回放使用。
 */
public class ReplayMockSupport {

	// runs before the mock is applied
	private static final long REAL_START_MILLIS = System.currentTimeMillis();//真实的初始化时间
	private static final long REAL_START_NANOS = System.nanoTime();
	/**
	 * 模拟的相对时间偏移量，单位ms，负数表示过去，正数表示未来，0表示使用当前时间
	 */
	private static volatile long OFFSET_MILLIS = 0L;

	static{
		if(FlowReplayUtils.isReplayEnv()){
			Startup.premain(null, ByteBuddyAgent.install());
			init();
		}
	}

	private static void init() {
		MockUp<System> systemMockUp = new MockUp<System>(){
			//覆盖System同名方法
			@Mock
			public long currentTimeMillis(){
				return millisSinceClassInit() + REAL_START_MILLIS + OFFSET_MILLIS;
			}

		};
		MockUp<UUID> uuidMockUp = new MockUp<UUID>(){
			//覆盖UUID同名方法
			@Mock
			public String toString(){
				return FlowReplayConstants.DEFAULT_UUID;
			}

		};
	}

	/**
	 * 从系统启动到当前经过的真实ms数
	 * @return
	 */
	private static long millisSinceClassInit() {
		return (System.nanoTime() - REAL_START_NANOS) / 1000000;
	}

	/**
	 * 设置时间偏移量，单位ms，比如这里设置 -1000，则每次调用System.currentTimeMillis()返回的时间都比真实时间少1秒. 设置为0则没有偏移
	 * @param offsetMillis
	 */
	public static void setOffsetMillis(long offsetMillis){
		ReplayMockSupport.OFFSET_MILLIS = offsetMillis;
	}

}

==> spring-boot-ext-1.3.14.8-sources.jar: cn/com/duibaboot/ext/autoconfigure/flowreplay/replay/ReplayMockSupport.java <==
package cn.com.duibaboot.ext.autoconfigure.flowreplay.replay;

import cn.com.duibaboot.ext.autoconfigure.flowreplay.FlowReplayConstants;
import cn.com.duibaboot.ext.autoconfigure.flowreplay.FlowReplayUtils;
import lombok.extern.slf4j.Slf4j;
import mockit.Mock;
import mockit.MockUp;
import mockit.internal.startup.Startup;
import net.bytebuddy.agent.ByteBuddyAgent;

import java.util.UUID;

/**
 * 此类用于返回模拟的方法值，目前用于 System.currentTimeMillis()、new Date()、UUID.randomUUID().toString() 等调用，这个类仅供回归使用。
 */
@Slf4j
public class ReplayMockSupport {

	// runs before the mock is applied
	private static final long REAL_START_MILLIS = System.currentTimeMillis();//真实的初始化时间
	private static final long REAL_START_NANOS = System.nanoTime();
	/**
	 * 模拟的相对时间偏移量，单位ms，负数表示过去，正数表示未来，0表示使用当前时间
	 */
	private static volatile long OFFSET_MILLIS = 0L;

	static{
		if(FlowReplayUtils.isReplayEnv()){
			Startup.premain(null, ByteBuddyAgent.install());
			init();
		}
	}

	private static void init() {
		MockUp<System> systemMockUp = new MockUp<System>(){
			//覆盖System同名方法
			@Mock
			public long currentTimeMillis(){
				return millisSinceClassInit() + REAL_START_MILLIS + OFFSET_MILLIS;
			}

		};
		MockUp<UUID> uuidMockUp = new MockUp<UUID>(){
			//覆盖UUID同名方法
			@Mock
			public String toString(){
				return FlowReplayConstants.DEFAULT_UUID;
			}

		};
	}

	/**
	 * 从系统启动到当前经过的真实ms数
	 * @return
	 */
	private static long millisSinceClassInit() {
		return (System.nanoTime() - REAL_START_NANOS) / 1000000;
	}

	/**
	 * 设置时间偏移量，单位ms，比如这里设置 -1000，则每次调用System.currentTimeMillis()返回的时间都比真实时间少1秒. 设置为0则没有偏移
	 * @param offsetMillis
	 */
	public static void setOffsetMillis(long offsetMillis){
		log.warn("setOffsetMillis被执行,offsetMillis={}", offsetMillis, new Exception());
		ReplayMockSupport.OFFSET_MILLIS = offsetMillis;
	}

}
